// =========================================================================================================
//  Mobile Target Range
//  Version: 2.8.0
//  Author: Kronzky (www.kronzky.info / kronzky@gmail.com)
//
// =========================================================================================================
//
//  Creates renewable targets at a defined range, and provides ballistic information
//
// =========================================================================================================
//
//  Mandatory Parameters:
//                   nul=[] exec "targetrange.sqf"
//
//  Optional Parameters:
//                nul=[[targets],stepsize,movespeed,movedist,mindist] execVM "targetrange.sqf"
//          e.g.  nul=[["SoldierEB",0,0,"T72"],"mindist:",250] execVM "targetrange.sqf"
//
//     targets  : Array of targets to be used. 
//                Array entries can be either objects that have already been placed on the map,
//                a class name, or a zero (in that case a gap is left at that position).
//
//     run      : If AI units have been defined as targets, they can be made to run normally.
//     ("")       Enter as "run:<speedmode>", e.g. [...,"run:full",...]     
//                   
//     movespeed: If > 0 then targets will move left and right (1=slowest, 10=fastest)
//     (0)        Enter as "speed:<value>", e.g. [...,"speed:2",...]     
//                   
//     movedist : How far moving targets should move
//     (0)        Enter as "move:<value>", e.g. [...,"move:10",...]     
//                 
//     defdist : Default distance for targets 
//     (100)     Enter as "defdist:<value>", e.g. [...,"defdist:500",...]     
//                 
//     Only the position of the target array is critical (has to be the first argument).
//     All other options can be defined in any position.
//  
// =========================================================================================================
//
//  After each shot the distance reached (and average speed) is displayed in the lower left corner.
//  To access the "bullet cam" (which follows the fired projectile), or slow-mo mode, use the radio menu.
//  A "stray" bullet cam can be aborted in-flight by activating radio code 0-0-0.
//
// =========================================================================================================
#include "KRON_Strings.sqf";
#include "functions.sqf";
sleep .1;
player setCaptive true;

KRON_orgPos=getPos player;
KRON_orgDir=getDir (vehicle player);
KRON_moved=false;
// re-position targets
_move=false;
if (count _this>0) then {
	if (typeName (_this select 0)=="OBJECT") then {_move=true};
};


// reposition targets
if (_move) exitWith {
	_distmod = _this select 3;
	KRON_dist = KRON_dist + _distmod;
	titleText [format["Current distance: %1 meters",KRON_dist],"plain down"];
	for "_i" from 0 to (count KRON_targets)-1 do {
		_target = KRON_targets select _i;
		if (typename _target=="OBJECT") then {
			if ((_target isKindOf "Man") && (KRON_run!="")) then {
				[_target] call _setWP;
			};
			_pos=[KRON_orgPos, KRON_orgDir, [_i,GAP_WIDTH] call KRON_xOff, KRON_dist] call KRON_rotpoint;
			_target setPos [_pos select 0, _pos select 1, 0];
			_target setDir ([KRON_orgDir,(KRON_targetType select _i) select 1] call KRON_addangle);
		};
	};
	KRON_moved=true;
	call _makeWalls;
};

// target movement definitions
KRON_MoveDist = parseNumber([_this,"move","10"] call _getArg);
KRON_MoveSpeed = parseNumber([_this,"speed","0"] call _getArg);
KRON_Move = (KRON_MoveSpeed>0);

// default target distance (keep "mindist" check for backward compatibility)
KRON_dist=parseNumber([_this,"mindist","0"] call _getArg);
if (KRON_dist==0) then {
	KRON_dist=parseNumber([_this,"defdist","100"] call _getArg);
};
KRON_dist max 100;

// have AI targets run
KRON_run=[_this,"run",""] call _getArg;
if (KRON_run!="") then {
	KRON_Move=true;
};

// create list of target types
KRON_targetType = [];
if ((typeName (_this select 0))=="array") then {
	{
		if (typeName _x=="object") then {
				KRON_targetType = KRON_targetType + [[typeOf _x,direction _x]];
				deleteVehicle _x;
		} else {
			_dir=0;
			_cls=_x;
			if (typeName _cls=="string") then {
				if (isClass (configFile >> "cfgVehicles" >> _cls)) then {
					if (_cls isKindOf "Man") then {
						_dir=180;
					};
				} else {
					_cls=0;
				};
			};
			KRON_targetType = KRON_targetType + [[_cls,_dir]];
		};
	}forEach (_this select 0);
};
if (count KRON_targetType == 0) then {KRON_targetType = [["TargetEpopup",0]]};
if ((typeName KRON_targetType!="array") || (count KRON_array<1)) exitWith {hint "No targets defined"};
KRON_Cnt=0;

// have a group for opfor targets
_opfor=switch (side player) do {case west:{east}; case east:{west}; default {east}};
createCenter _opfor;
KRON_grp=createGroup _opfor;

// create targets objects
KRON_targets=[];
KRON_unitpos=[];
_stances=["up","middle","down"]; 
_stanceidx=0;
for "_i" from 0 to (count KRON_targetType)-1 do {
	_type = (KRON_targetType select _i) select 0;
	if (typeName _type=="STRING") then {
		if (_type isKindOf "Man") then {
			KRON_unitpos set [_i,_stances select _stanceidx];
			if (KRON_run=="") then {
			_stanceidx=_stanceidx+1;
			if (_stanceidx>=count _stances) then {_stanceidx=0};
			};
		} else {
			KRON_unitpos set [_i,0];
			_isvehicle=false;
		};
		_obj=[_i] call KRON_maketarget;
	} else {
		KRON_targets set [_i,0]; 
	};
};

// create marker walls & flags
call _makeWalls;

// create radio calls
KRON_BulletCam=0;
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Alpha","PRESENT",true];
_radio setTriggerStatements["this","KRON_BulletCam=0;",""];
1 setRadioMsg "BulletCam off";
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Bravo","PRESENT",true];
_radio setTriggerStatements["this","KRON_BulletCam=1.00;",""];
2 setRadioMsg "BulletCam on (acc: 1.00)";
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Charlie","PRESENT",true];
_radio setTriggerStatements["this","KRON_BulletCam=.10;",""];
3 setRadioMsg "BulletCam on (acc: 0.10)";
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Delta","PRESENT",true];
_radio setTriggerStatements["this","KRON_BulletCam=.01;",""];
4 setRadioMsg "BulletCam on (acc: 0.01)";
KRON_SlowMo=1;
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Echo","PRESENT",true];
_radio setTriggerStatements["this","if (KRON_SlowMo>.5) then {5 setRadioMsg 'Slow-Mo off'; setAccTime .1; KRON_SlowMo=.1} else {5 setRadioMsg 'Slow-Mo on'; setAccTime 1; KRON_SlowMo=1};",""];
5 setRadioMsg 'Slow-Mo on';
KRON_Marker=false;
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Foxtrot","PRESENT",true];
_radio setTriggerStatements["this","6 setRadioMsg (if (KRON_Marker) then {'Destination marker off'} else {'Destination marker on'}); KRON_Marker=!(KRON_Marker);",""];
6 setRadioMsg "Destination marker on";
KRON_Ballistics=false;
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Golf","PRESENT",true];
_radio setTriggerStatements["this","hint ''; 7 setRadioMsg (if (KRON_Ballistics) then {'Ballistics on'} else {'Ballistics off'}); KRON_Ballistics=!(KRON_Ballistics); if (KRON_Ballistics) then {KRON_BulletCam=0}",""];
7 setRadioMsg "Ballistics on";
_ballistics=createTrigger["EmptyDetector",[0,0]];
_ballistics setTriggerActivation["NONE","PRESENT",true];
_ballistics setTriggerStatements["format['%1',ballloop]!='false'","wdir=[] call KRON_weapondir; ballloop=false; if (KRON_Ballistics && !dialog) then {hint format['elev: %1',wdir]}","ballloop=true"];
if (KRON_Move) then {
	_radio=createTrigger["EmptyDetector",[0,0]];
	_radio setTriggerActivation["Hotel","PRESENT",true];
	_radio setTriggerStatements["this","8 setRadioMsg (if (KRON_Move) then {'Movement on'} else {'Movement off'}); KRON_Move=!(KRON_Move); {if (_x!=player) then {_x stop !KRON_Move}}forEach allUnits",""];
	8 setRadioMsg "Movement off";
};
KRON_AbortCam=false;
_radio=createTrigger["EmptyDetector",[0,0]];
_radio setTriggerActivation["Juliet","PRESENT",true];
_radio setTriggerStatements["this","KRON_AbortCam=true;",""];
10 setRadioMsg "Abort BulletCam";


// assign fired event handlers to player & all vehicles
// create action menu
{
	_x addAction ['Re-align with player', 'targetrange.sqf', 0, 1, true, true, "", "_target==vehicle _this"];
	_x addAction ['Increase distance by 50m', 'targetrange.sqf', 50, .9, true, true, "", "(KRON_dist>=100) && (_target==vehicle _this)"];
	_x addAction ['Decrease distance by 50m', 'targetrange.sqf', -50, .8, true, true, "", "(KRON_dist>=100) && (_target==vehicle _this)"];
	_x addAction ['Increase distance by 10m', 'targetrange.sqf', 10, .9, true, true, "", "(KRON_dist<100) && (_target==vehicle _this)"];
	_x addAction ['Decrease distance by 10m', 'targetrange.sqf', -10, .8, true, true, "", "(KRON_dist<100) && (KRON_dist>10) && (_target==vehicle _this)"];
	
	_x addEventHandler["fired","if !(KRON_tracking) then {setAccTime .001; KRON_weapon=_this select 1; _this execVM 'bullettrack.sqf'}"];
}forEach [player]+vehicles;
player addAction ["Select Weapon", "selectweapon.sqf",0,.7];

titleText["Use action menu to change distance","plain down"];

// "HQ" sideLogic for globalChats
createCenter sideLogic;
_grpLogic = createGroup sideLogic;
KRON_HQ = _grpLogic createUnit ["Logic", [0,0,0], [], 0, "NONE"];

// global flag for bullet tracking
KRON_tracking=false;

// constantly move targets back & forth, and re-supply player with ammo
_movestep=.01*KRON_MoveSpeed;
_orgpos=[];
_moveoff=0;
_movedir=1;
_pdiff=0;
_cycles=0;
KRON_moved=true;
KRON_MAGS=magazines player;
while {true} do {
	if (KRON_MoveSpeed>0) then {
		// targets were repositioned
		if (KRON_moved) then {
			for "_u" from 0 to (count KRON_targets)-1 do {
				_unit = KRON_targets select _u;
				if (typename _unit=="object") then {
					_orgpos set [_u,getPos _unit];
				};
			};
			_moveoff=0;
			_movedir=1;
			_pdiff=0;
			KRON_moved=false;
		};
		if (KRON_Move) then {
			_moveoff=_moveoff+_movedir*_movestep;
			if (abs(_moveoff)>KRON_MoveDist)	then {_movedir=_movedir*-1};
			_pdiff=_pdiff+(_movedir*_movestep);
		};
		for "_u" from 0 to (count KRON_targets)-1 do {
			_unit = KRON_targets select _u;
			if (typename _unit=="object") then {
				if (!(_unit isKindOf "Man") || (KRON_run=="")) then {
				_pos=[_orgpos select _u, KRON_orgDir, _pdiff, 0] call KRON_rotpoint;
				_unit setPos _pos;
				};
			}
		};
	};
	sleep .01;
	_cycles=_cycles+.01;
	
	// give player infinite ammo (without reloading needed)
	if (_cycles>.5) then {
	  (vehicle player) setvehicleammo 1;
		_mags = magazines player;
		{
			if !(_x in _mags) then {
				player addMagazine _x;
			};
		}forEach KRON_MAGS;
		
		// update wall visibility (for map view)
		{player reveal _x}forEach KRON_walls;
		_cycles=0;
	};	
};
